@@ -25,6 +25,14 @@ class EditorField
   var $LookupSql;
   var $OptionList;
 
+  /**
+   * Creates an EditorField for use in the Editor, possibly initialising the SQL for calculating it's
+   * value, and lookup_sql for use in drop-down lists.
+   *
+   * @param unknown $field
+   * @param string $sql
+   * @param string $lookup_sql
+   */
   function __construct( $field, $sql="", $lookup_sql="" ) {
     global $session;
     $this->Field      = $field;
@@ -37,14 +45,37 @@ class EditorField
     $this->Value = $value;
   }
 
+  /**
+   * Set the SQL used for this field, if it is more than just a field name.
+   * @param unknown $sql
+   */
   function SetSql( $sql ) {
     $this->Sql  = $sql;
   }
 
+  /**
+   * Set the lookup SQL to use to populate a SELECT for this field.
+   * @param string $lookup_sql
+   */
   function SetLookup( $lookup_sql ) {
     $this->LookupSql  = $lookup_sql;
   }
 
+  /**
+   * Set the SELECT values explicitly, if they are not available in SQL.
+   *
+   * For example:
+   *
+   *   SetOptionList(array('M' => 'Male', 'F' => 'Female', 'O' => 'Other'), 'F', array('maxwidth' => 6, 'translate' => true));
+   *
+   * This would present Male/Female/Other drop-down, when in another language the values
+   * would be translated (if available), e.g. in German as Männlich/Weiblich/Andere, except
+   * that in this case Männli/Weibli/Andere, since the values would be truncated to maxwidth.
+   *
+   * @param array $options An array of key => value pairs
+   * @param string $current The currently selected key
+   * @param string $parameters An array of parameters (maxwidth & translate are the only valid parameters)
+   */
   function SetOptionList( $options, $current = null, $parameters = null) {
     if ( gettype($options) == 'array' ) {
       $this->OptionList = '';
@@ -76,21 +107,48 @@ class EditorField
     return "$this->Sql AS $this->Field";
   }
 
+  /**
+   * Add some kind of attribute to this field, such as a 'class' => 'fancyinputthingy'
+   *
+   * @param string $k The attribute name
+   * @param string $v The attribute value
+   */
   function AddAttribute( $k, $v ) {
     $this->Attributes[$k] = $v;
   }
 
+  /**
+   * Render a LABEL around something.  In particular it is useful to render a label around checkbox fields to include their labels and make them clickable.
+   *
+   * The label value itself must be in the '_label' attribute, and the field must also have an 'id' attribute.
+   *
+   * @param string $wrapme The rendered field to be wrapped
+   * @return string
+   */
+  function RenderLabel( $wrapme ) {
+    if ( !isset($this->Attributes['_label']) || !isset($this->Attributes['id'])) return $wrapme;
+    $class = (isset($this->Attributes['class']) ? $this->Attributes['class'] : 'entry');
+    $title = (isset($this->Attributes['title']) ? ' title="'.str_replace('"', '&#39;', $this->Attributes['title']) . '"' : '');
+    return( sprintf( '<label for="%s" class="%s"%s>%s %s</label>',
+             $this->Attributes['id'], $class, $title, $wrapme, $this->Attributes['_label']) );
+  }
+
+  /**
+   * Render the array of attributes for inclusion in the input tag.
+   * @return string
+   */
   function RenderAttributes() {
     $attributes = "";
     if ( count($this->Attributes) == 0 ) return $attributes;
     foreach( $this->Attributes AS $k => $v ) {
+      if ( $k == '_label' ) continue;
       $attributes .= " $k=\"" . str_replace('"', '&#39;', $v) . '"';
     }
     return $attributes;
   }
 
-  
-  
+
+
 }
 
 
@@ -118,6 +176,16 @@ class Editor
   var $SubmitName;
   var $Id;
 
+  /**
+   * Constructs an editor widget, with a title and fields.
+   *
+   * The second parameter maybe passed as a string, to be interpreted as the name of a table, from
+   * which all fields will be included, or as an array of specific fields, in which case you should
+   * make sure to call SetBaseTable('tablename') so the editor knows where to find those fields!
+   *
+   * @param string $title
+   * @param array or string $fields See above
+   */
   function __construct( $title = "", $fields = null ) {
     global $c, $session, $form_id_increment;
     $this->Title = $title;
@@ -147,16 +215,34 @@ class Editor
     @dbg_error_log( 'editor', 'DBG: New editor called %s', $title);
   }
 
+  /**
+   * Creates a new field in the Editor, possibly initialising the SQL for calculating it's
+   * value, and lookup_sql for use in drop-down lists.
+   *
+   * @param string $field The name for the field.
+   * @param string $sql The SQL for the target list. Think: "$sql AS $field"
+   * @param string $lookup_sql The SQL for looking up a list of possible stored values and displayed values.
+   */
   function &AddField( $field, $sql="", $lookup_sql="" ) {
     $this->Fields[$field] = new EditorField( $field, $sql, $lookup_sql );
     $this->OrderedFields[] = $field;
     return $this->Fields[$field];
   }
 
+  /**
+   * Set the SQL for this field for the target list.  Think: "$sql AS $field"
+   * @param string $field
+   * @param string $sql
+   */
   function SetSql( $field, $sql ) {
     $this->Fields[$field]->SetSql( $sql );
   }
 
+  /**
+   * Set the SQL for looking up a list of possible stored values and displayed values.
+   * @param string $field
+   * @param string $lookup_sql
+   */
   function SetLookup( $field, $lookup_sql ) {
     if (is_object($this->Fields[$field])) {
       $this->Fields[$field]->SetLookup( $lookup_sql );
@@ -191,26 +277,49 @@ class Editor
     return $this->Id;
   }
 
+  /**
+   * Set the explicit options & parameters for a list of stored/displayed values.  See the
+   * description under EditorField::SetOptionList() for full details.
+   *
+   * @param string $field
+   * @param array $options A key => value array of valid store => display values.
+   * @param string $current The key of the current row
+   * @param string $parameters Set maxwidth & whether displayed values are translated.
+   */
   function SetOptionList( $field, $options, $current = null, $parameters = null) {
     $this->Fields[$field]->SetOptionList( $options, $current, $parameters );
   }
 
+  /**
+   * Add an attribute to this field.
+   * @param unknown $field
+   * @param unknown $k
+   * @param unknown $v
+   */
   function AddAttribute( $field, $k, $v ) {
     $this->Fields[$field]->AddAttribute($k,$v);
 
   }
 
+  /**
+   * Set the base table for the row query.
+   * @param unknown $base_table
+   */
   function SetBaseTable( $base_table ) {
     $this->BaseTable = $base_table;
   }
 
+  /**
+   * Set any joins
+   * @param unknown $join_list
+   */
   function SetJoins( $join_list ) {
     $this->Joins = $join_list;
   }
 
 
   /**
-  * Accessor for the Title for the browse, which could set the title also.
+  * Accessor for the Title for the editor, which could set the title also.
   *
   * @param string $new_title The new title for the browser
   * @return string The current title for the browser
@@ -221,6 +330,10 @@ class Editor
   }
 
 
+  /**
+   * Set the name of the SUBMIT button
+   * @param unknown $new_submit
+   */
   function SetSubmitName( $new_submit ) {
     $this->SubmitName = $new_submit;
   }
@@ -229,6 +342,10 @@ class Editor
     return isset($_POST[$this->SubmitName]);
   }
 
+  /**
+   * Magically knows whether you are in the processing the result of an update or a create.
+   * @return boolean
+   */
   function IsUpdate() {
     $is_update = $this->Available();
     if ( isset( $_POST['_editor_action']) && isset( $_POST['_editor_action'][$this->Id]) ) {
@@ -238,18 +355,35 @@ class Editor
     return $is_update;
   }
 
+  /**
+   * The opposite of IsUpdate.  Really.
+   * @return boolean
+   */
   function IsCreate() {
     return ! $this->IsUpdate();
   }
 
+  /**
+   * Set the row selection criteria
+   * @param unknown $where_clause
+   */
   function SetWhere( $where_clause ) {
     $this->Where = $where_clause;
   }
 
+  /**
+   * Set the criteria used to find the new row after it got created.
+   * @param unknown $where_clause
+   */
   function WhereNewRecord( $where_clause ) {
     $this->NewWhere = $where_clause;
   }
 
+  /**
+   * Append more stuff to the WHERE clause
+   * @param unknown $operator
+   * @param unknown $more_where
+   */
   function MoreWhere( $operator, $more_where ) {
     if ( $this->Where == "" ) {
       $this->Where = $more_where;
@@ -266,20 +400,45 @@ class Editor
     $this->MoreWhere("OR",$more_where);
   }
 
+  /**
+   * Set this to be the form display template.  It's better to use Layout($template) in general.
+   *
+   * @deprecated
+   * @param string $template
+   */
   function SetTemplate( $template ) {
+    deprecated('Editor::SetTemplate');
     $this->Template = $template;
   }
 
+  /**
+   * Like SetTemplate($template) except it surrounds the template with a ##form## ... </form> if
+   * there is not a form already in the template.
+   *
+   * @param string $template
+   */
   function Layout( $template ) {
-    if ( strstr( $template, '##form##' ) === false && stristr( $template, '<form' ) === false ) $template = '##form##' . $template;
+    if ( strstr( $template, '##form##' ) === false && stristr( $template, '<form' ) === false )
+      $template = '##form##' . $template;
     if ( stristr( $template, '</form' ) === false ) $template .= '</form>';
     $this->Template = $template;
   }
 
+  /**
+   * Returns 'true' if we have read a row from the database (or set one through SetRecord()), 'false' otherwise.
+   *
+   * @return boolean
+   */
   function Available( ) {
     return $this->RecordAvailable;
   }
 
+  /**
+   * Set a database row to load the field values from.
+   *
+   * @param object $row
+   * @return object The row that was passed in.
+   */
   function SetRecord( $row ) {
     $this->Record = $row;
     $this->RecordAvailable = is_object($this->Record);
@@ -314,6 +473,12 @@ class Editor
     }
   }
 
+  /**
+   * Read the record from the database, optionally overriding the WHERE clause.
+   *
+   * @param string $where (optional) An SQL WHERE clause to override any previous SetWhere call.
+   * @return object The row that was read from the database.
+   */
   function GetRecord( $where = "" ) {
     global $session;
     $target_fields = "";
@@ -338,7 +503,33 @@ class Editor
 
 
   /**
-  * Replace parts into the form template.
+  * Replace parts into the form template.  Parts that are replaceable are listed below:
+  *   ##form##        A <form ...> tag.  You should close this with </form> or use Layout($template) which will take care of it for you.
+  *   ##submit##      A <input type="submit" ...> tag for the form.
+  *   ##f.options##   A list of options explicitly specified
+  *   ##f.select##    A select list from the lookup SQL specified
+  *   ##f.checkbox##  A checkbox, perhaps with a "_label" attribute
+  *   ##f.input##     A normal input field.
+  *   ##f.file##      A file upload field.
+  *   ##f.money##     A money input field.
+  *   ##f.date##      A date input field.
+  *   ##f.textarea##  A textarea
+  *   ##f.hidden##    A hidden input field
+  *   ##f.password##  An input field for entering passwords without them being echoed to the screen
+  *   ##f.enc##       Just print the value with special chars escaped for use in URLs.
+  *   ##f.submit##    An <input type="submit" where you specify the field name.
+  *
+  * Most of these begin with "f", which should be replaced by the name of the field.  Many also take an option
+  * after the name as well, so (for example) you can force the current value in ##options## or ##select## by
+  * setting ##field.select.current##.  The input, file, money & date all accept the third parameter as a size
+  * value, so ##fieldname.date.14## would be a 14-character-wide date field. Similarly a textarea allows for
+  * a COLSxROWS value, so ##myfield.textarea.80x5## would be an 80-column textarea, five rows high.
+  *
+  * For ##fieldname.password.fakevalue## you can set the 'fake' value used to populate the password field so
+  * that you can check for this on submit to be able to tell whether the password field has been edited.
+  *
+  * Other attributes are added to the <input ...> tag based on any SetAttributes() that may have been applied.
+  *
   * @param array $matches The matches found which preg_replace_callback is calling us for.
   * @return string What we want to replace this match with.
   */
@@ -360,7 +551,7 @@ class Editor
     // $matches[1] the match for the first subpattern
     // enclosed in '(...)' and so on
     $field_name = $matches[1];
-    $what_part = $matches[3];
+    $what_part = (isset($matches[3]) ? $matches[3] : null);
     $part3 = (isset($matches[5]) ? $matches[5] : null);
 
     $value_field_name = $field_name;
@@ -408,6 +599,10 @@ class Editor
         }
         return '<select class="entry" name="'.$field_name.'"'.$attributes.'>'.$option_list.'</select>';
       case "checkbox":
+        if ( !isset($field) ) {
+          @dbg_error_log("ERROR","Field '$field_name' is not defined.");
+          return "<p>Error: '$field_name' is not defined.</p>";
+        }
         if ( $field_value === true ) {
           $checked = ' CHECKED';
         }
@@ -420,12 +615,12 @@ class Editor
             case '0':
               $checked = "";
               break;
-  
+
             default:
               $checked = ' CHECKED';
           }
         }
-        return '<input type="hidden" value="off" name="'.$field_name.'"><input class="entry" type="checkbox" value="on" name="'.$field_name.'"'.$checked.$attributes.'>';
+        return $field->RenderLabel('<input type="hidden" value="off" name="'.$field_name.'"><input class="entry" type="checkbox" value="on" name="'.$field_name.'"'.$checked.$attributes.'>' );
       case "input":
         $size = (isset($part3) ? $part3 : 6);
         return "<input class=\"entry\" value=\"".htmlspecialchars($field_value)."\" name=\"$field_name\" size=\"$size\"$attributes>";
@@ -451,7 +646,7 @@ class Editor
       case "submit":
         $action =  ( $this->RecordAvailable ? 'update' : 'insert' );
         return sprintf('<input type="hidden" name="_editor_action[%s]" value="%s"><input type="submit" class="submit" name="%s" value="%s">',
-                                                              $this->Id, $action,                           $this->SubmitName, $value_field_name );
+                                                              $this->Id, $action, $this->SubmitName, $value_field_name );
       default:
         return str_replace( "\n", "<br />", $field_value );
     }
@@ -479,9 +674,8 @@ class Editor
   }
 
   /**
-  * Write the record
-  * @param boolean $is_update Tell the write whether it's an update or insert.  Hopefully it
-  * should be able to figure it out though.
+  * Write the record.  You might want to consider calling Editor::WhereNewRecord() before this if it might be creating a new record.
+  * @param boolean $is_update Explicitly tell the write whether it's an update or insert.  Generally it should be able to figure it out though.
   */
   function Write( $is_update = null ) {
     global $c, $component;
